/*
 * Copyright (c) 2025. Sergey S. Chernov - All Rights Reserved
 *
 *  You may use, distribute and modify this code under the
 * terms of the private license, which you must obtain from the author
 *
 * To obtain the license, contact the author: https://t.me/real_sergeych or email to
 * real dot sergeych at gmail.
 */

package net.sergeych.kiloparsec

import io.ktor.utils.io.*

object AsyncVarint {
    suspend fun encodeUnsigned(value: ULong, output: ByteWriteChannel) {
        var rest = value
        do {
            val x = (rest and 127u).toInt()
            rest = rest shr 7
            if (rest > 0u)
                output.writeByte(x or 0x80)
            else
                output.writeByte(x)

        } while (rest > 0u)
    }

    suspend fun decodeUnsigned(source: ByteReadChannel): ULong {
        var result: ULong = 0u
        var count = 0
        while (true) {
            val x = source.readUByte().toInt()
            result = result or ((x and 0x7F).toULong() shl count)
            if ((x and 0x80) == 0)
                break
            count += 7
        }
        return result
    }
}

suspend fun ByteReadChannel.readUByte(): UByte = this.readByte().toUByte()

suspend fun ByteWriteChannel.writeByte(byte: Int) {
    this.writeByte(byte.toByte())
}
